require 'redis'

module Backrub
  module Store
    class Redis < Base
      attr_reader :config, :backlog_size

      def initialize(config={}, backlog_size=100)
        @config = config
        @backlog_size = backlog_size
      end

      def redis
        @redis ||= ::Redis.new(config)
      end

      def backlog(channel, count)
        backlog = redis.lrange(channel, 0, count - 1)

        backlog.reverse_each do |message|
          yield channel, message
        end
      end

      def publish(channel, message)
        redis.multi do
          redis.publish(channel, message)
          redis.lpush(channel, message)
          redis.ltrim(channel, 0, backlog_size - 1)
        end
      end

      def subscribe(*channels)
        # Open a new connection because the connection blocks, causing other threads to be unable to use it
        local_redis = ::Redis.new(config)
        local_redis.subscribe(*channels) do |on|
          on.message do |channel, message|
            yield channel, message
          end
        end
      ensure
        local_redis.quit
      end
    end
  end
end
